//TT UI Builder


//TT_UI_API functions mock
if( typeof(setDialogSize) !== "function" ) {

    var setDialogSize = function (width, height) {
		},
        getGuiLanguage = function () {
            return "de-de";
        },
        getSharedPath = function () {
            return "";
        },
        getParameterList = function () {
            return '["url", "id"]';
        },
        getParameterValue = function () {
            return "true";
        },
        getParameterValueFromModelObject = function () {
            return "true";
        },
        setParameterValue = function (id, value) {

		},
        importFile = function (id) {
            return "test.html";
        },
        saveChanges = function () {
        },
        closeDialog = function () {
        };
}

// In IE 9 or less, the console object doesn't exist
if (!window.console) {
    window.console = {log: function() {}};
}

// deactivate contextmenu
$(document).ready(function () {

    $(document).bind("contextmenu", function(){
        return false;
    });

    $(document).bind("keydown keyup", function(e){

        var keyCode = e.which;

        // avoid F-Keys
        if( (keyCode >= 112) && (keyCode <= 123) ) {
            e.returnValue = false;
            e.cancelBubble = true;
            return false;
        }

        return true;
    });
});



var ValueUtil = {

        converters : {},

        getConverterName : function( id ){

            if( !id || id.length === 0 ) {
               return "defaultConverter";
            }

            return id.replace( /\./gi, "_");
        },

        registerConverter : function( id, converter ){

            if( !converter || typeof(converter.convertValueOnGet) !== "function" || typeof(converter.convertValueOnSet) !== "function" ){
                return;
            }

            var converterName = this.getConverterName(id);
            this.converters[ converterName ] = converter;
        },
        get: function (id) {

            var value = getParameterValue(id),
                converterName = this.getConverterName(id);

            if( typeof(this.converters[converterName]) !== "undefined" ){
                value = this.converters[converterName].convertValueOnGet( value );
            }

            return ( value );
        },
        getFromModelObject: function (id) {
            var value = getParameterValueFromModelObject( id ), // browser function
                converterName = this.getConverterName(id);

            if( typeof(this.converters[converterName]) !== "undefined" ){
                value = this.converters[converterName].convertValueOnGet( value );
            }

            return ( value );
        },

        set: function (id, value) {

            if (!value) {
                value = "";
            }

            var converterName = this.getConverterName(id);
            if( typeof(this.converters[converterName]) !== "undefined" ){
                value = this.converters[converterName].convertValueOnSet( value );
            }

            setParameterValue(id, value);
        },
        getList : function() {
            return getParameterList();
        }
    },
    getValue = function( value, defaultValue ) {
        return ((typeof(value) !== 'undefined') && (value !== null)) ? value : defaultValue;
    };

var TT_UI_Builder = function (tableBodyId, guiTexts) {

    this.builder.initialize(tableBodyId, guiTexts);

    this.customizing.initialize();

    this.defaultSetup = function () {
        this.assignCaption("caption");
        this.assignOkButton("okButton");
        this.assignApplyButton("applyButton");
        this.assignCancelButton("cancelButton");
        this.addEditEntry("com.tts.description");
        this.assignCaption("componentHelp");
    };

    this.registerValueConverter = function( id, converter ){
        ValueUtil.registerConverter( id, converter );
    };

	this.initializeTabs = function() {
		var builder = this;

		$("#tabs").tabs();

		$.each(tableBodyId, function(key){
			$('#menu_' + key).text( builder.getLabel(key) );
		});
	};

	this.initializeHelpButton = function() {
		var $componentHelp = $("#componentHelp");
		$componentHelp.hide();
		$("#header")
			.click(function () {
				if ($componentHelp.is(':visible')) {
					$componentHelp.hide();
				} else {
					$componentHelp.slideDown();
				}
			})
			.css('cursor', 'pointer');
	};

	this.initializeSlider = function() {
		var customizing = this.customizing;

		$(".sliderSingle").each(function () {

            var slider = $(this),
			    id = slider.attr("id");

			slider.slider({
				value: (customizing.getValue(id, 'value', function(){ return 0; }))(),
				min: customizing.getValue(id, 'minval', 0),
				max: customizing.getValue(id, 'maxval', 10),
				step: customizing.getValue(id, 'step', 1),
				slide: customizing.getValue(id, 'slide', function () {})
			}).each(function () {

				var opt = slider.data().uiSlider.options,
					vals = opt.max - opt.min,
					increment = Math.max(1, Math.floor(vals / customizing.getValue(id, 'nrLabels', 10))),
					generateLabel = customizing.getValue(id, 'genLabel', function(currentValue) {
						return currentValue;
					} );

				for (var i = 0; i <= vals; i += increment) {
					// Create a new element and position it with percentages
					var el = $('<label>' + generateLabel(i + opt.min) + '</label>').css({
						'left': (i / vals * 100) + '%'
					});

					// Add the element inside #slider
					slider.append(el);
				}
			});
			if (slider.attr("permitted") === "false") {
				slider.slider("disable");
			}
		});
		$(".sliderRanged").each(function () {
			var slider = $(this),
			    id = slider.attr("id");

			slider.slider({
				range: true,
				values: (customizing.getValue(id, 'values', function(){ return [0,0]; }))(),
				min: customizing.getValue(id, 'minval', 0),
				max: customizing.getValue(id, 'maxval', 10),
				step: customizing.getValue(id, 'step', 1),
				slide: customizing.getValue(id, 'slide', function () {})
			}).each(function () {

				var opt = slider.data().uiSlider.options,
					vals = opt.max - opt.min,
					increment = Math.floor(vals / customizing.getValue(id, 'nrLabels', 10)),
					generateLabel = customizing.getValue(id, 'genLabel', function(currentValue) {
						return currentValue;
					} );

				for (var i = 0; i <= vals; i += increment) {
					// Create a new element and position it with percentages
					var el = $('<label>' + generateLabel(i + opt.min) + '</label>').css({
						'left': (i / vals * 100) + '%'
					});

					// Add the element inside #slider
					slider.append(el);
				}
			});

			if (slider.attr("permitted") === "false") {
				slider.slider("disable");
			}
		});
	};

	this.finalizeSetup = function () {

		this.initializeTabs();
		this.initializeHelpButton();
		this.initializeSlider();

        $.ajaxSetup({
            xhr: function()  {
                if ('ActiveXObject' in window) {
                    return new ActiveXObject("Microsoft.XMLHTTP");
                }
                else {
                    return new XMLHttpRequest();
                }
            }
        });

        $.get('help/' + getGuiLanguage() + '.htm', function (response) {
            $("#componentHelp").html(response);
        });

		// confirm dialog if on enter pressed
		$(document).keypress(function(e) {
			if(e.which === 13) {
				$('#okButton').trigger('click');
			}
			if(e.which === 27) {
				$('#cancelButton').trigger('click');
			}
		});
    };

    this.getImageSize = function( filePath, callback ) {

        var id = "tts-image-size-inspector",
            documentBody = $("body"),
            image = $(document.createElement("img"))
                .attr("id",id)
                .css({
                    "position" : "absolute",
                    "visibility" : "hidden"
                })
                .load( function(){
                    var img = $(this);
                    callback( img.width(), img.height() );
                    documentBody.remove( "#" + id );
                });

        documentBody.append( image );
        image.attr( "src", filePath );
    };

    this.getLabel = function (id) {
        return ( this.builder.textHandler.getCaption(id) );
    };

    this.assignCaption = function (elementId) {

        var captionText = this.getLabel(elementId);
        if (!captionText) {
            return;
        }

        var captionElement = $("#" + elementId);
        if (!captionElement) {
            return;
        }

        captionElement.html(captionText);
    };

    this.assignDescription = function (elementId) {

		var descriptionText = this.getLabel( elementId );
		if( !descriptionText ) {
			return;
		}

		var descriptionElement =  $( "#" + elementId );
		if( !descriptionElement ) {
			return;
		}

		descriptionElement.html( descriptionText );
    };

    this.assignOkButton = function (buttonId) {
        var okButton = $("#" + buttonId);
        okButton.attr("value", this.getLabel(buttonId));
        okButton.bind("click", function() {
            saveChanges();
            closeDialog();
        });
        okButton.button();
    };

    // for each element from jsonModifiedParamIDs that has an value-attribute
    // the value will refreshed with the value of the corresponding model object.
    function refreshParameters( jsonModifiedParamIDs ) {
        var paramArray = JSON.parse( jsonModifiedParamIDs );

        $(paramArray).each( function (i, id) {
            var elem = $("#" + id);
            if( elem && ( typeof elem.attr( "value" ) !== typeof undefined ) ) {
                elem.attr("value", ValueUtil.get( id ));
            }
        })
    }

    this.assignApplyButton = function (buttonId) {
        var applyButton = $("#" + buttonId);
        applyButton.attr("value", this.getLabel(buttonId));
        applyButton.bind("click", function() {
            refreshParameters( saveChanges() );
        });
        applyButton.button();
    };

    this.assignCancelButton = function (buttonId) {
        var cancelButton = $("#" + buttonId);
        cancelButton.attr("value", this.getLabel(buttonId));
        cancelButton.bind("click", function() {
            closeDialog();
        });
        cancelButton.button();
    };

    this.addSeperator = function (tabId) {
        this.builder.createSeperator(tabId);
    };

    this.addEntryDescription = function (id, tabId) {
        this.builder.createEntryDescription(tabId, id);
    };

// ein neue Funktion zum hinzufügen einer Beschreibung, die sich über die komplette Parameter-
// tabelle erstreckt.
    this.addDescription = function (id, tabId) {
        this.builder.createDescription(tabId, id);
    };

    this.addEditEntry = function (id, tabId) {
        var idCell = this.builder.createIdCell(id);
        var valueCell = this.builder.createEditCell(id);
        this.builder.createEntry(tabId, id, idCell, valueCell);
    };

    this.addMultiEntryHeader = function (slicedArgs, tabId) {
        var idCell = this.builder.createIdCell("");
        var valuesCell = this.builder.createMiniHeaderCell(slicedArgs);
        this.builder.createEntry(tabId, "", idCell, valuesCell);
    };

    this.addMultiEntry = function (label, prefix, slicedArgs, tabId) {
        var idCell = this.builder.createIdCell(label);
        var valuesCell = this.builder.createMiniEditCell(prefix, slicedArgs);
        this.builder.createEntry(tabId, label, idCell, valuesCell);
    };

    this.addTextAreaEntry = function (id, rows, cols, tabId) {
        var idCell = this.builder.createIdCell(id);
        var valueCell = this.builder.createTextAreaCell(id, rows, cols);
        this.builder.createEntry(tabId, id, idCell, valueCell);
    };

    this.addCompoBoxEntry = function (id, comboList, tabId) {
        if (!comboList) {
            return;
        }

        var idCell = this.builder.createIdCell(id);
        var valueCell = this.builder.createComboBoxCell(id, comboList);
        this.builder.createEntry(tabId, id, idCell, valueCell);
    };

    this.addLocalizedCompoBoxEntry = function (id, comboList, tabId) {
        if (!comboList) {
            return;
        }

        var idCell = this.builder.createIdCell(id);
        var valueCell = this.builder.createLocalizedComboBoxCell(id, comboList);
        this.builder.createEntry(tabId, id, idCell, valueCell);
    };

    this.addFileSelectionEntry = function (id, tabId, hideInfoButton, onSelectionCallback) {
        var idCell = this.builder.createIdCell(id);
        var valueCell = this.builder.createFileSelectionCell(id, hideInfoButton);
        this.builder.createEntry(tabId, id, idCell, valueCell);

        if( onSelectionCallback ) {
            $(window.document).on("file:selected", function (event, data) {
                if (id === data.id) {
                    onSelectionCallback(data.filePath);
                }
            });
        }
    };

    this.addCheckBoxEntry = function (id, tabId) {
        var idCell = this.builder.createIdCell(id);
        var valueCell = this.builder.createCheckBoxCell(id);
        this.builder.createEntry(tabId, id, idCell, valueCell);
    };

    this.addCustomEntry = function (id, element, tabId) {
        var idCell = this.builder.createIdCell(id);
        var valueCell = this.builder.createCustomCell(id, element);
        this.builder.createEntry(tabId, id, idCell, valueCell);
    };

    this.addSlideEntry = function (id, minValue, maxValue, step, tabId) {
        var idCell = this.builder.createIdCell(id);
        var localId = id.replace(/[^\w\d\-_.]/g, "_") + '_slider';
        var valueCell = this.builder.createSlider(localId);
        this.customizing
			.addEntry( localId , 'minval' , minValue )
			.addEntry( localId , 'maxval' , maxValue )
			.addEntry( localId , 'step' , step );

        this.builder.createEntry(tabId, localId, idCell, valueCell);
        return localId;
    };

    this.addRangeSlideEntry = function (id, minValue, maxValue, step, tabId) {
        var idCell = this.builder.createIdCell(id);
        var localId = id.replace(/[^\w\d\-_.]/g, "_") + '_sliderRanged';
        var valueCell = this.builder.createRangeSlider(localId);
        this.customizing.addEntry( localId , 'minval' , minValue ).addEntry( localId , 'maxval' , maxValue ).addEntry( localId , 'step' , step );
        this.builder.createEntry(tabId, localId, idCell, valueCell);

        return localId;
    };

	this.getValueLabelList = function(values) {
		var list = [],
			i,
			currentPointValue;

		for ( i=0; i < values.length; i++ ) {
			currentPointValue = values[i];
			list.push( { value: currentPointValue, label : currentPointValue.toString() } )
		}

		return list;
	};

	this.addFontSizeComboBoxEntry = function (id, tabId) {
		var myPoints = [5, 6, 7, 7.5, 8, 9,  10, 10.5, 11, 12, 13, 13.5, 14, 15, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 42, 60],
			myPixel =  [7, 8, 9, 10, 11, 12, 13, 14,   15, 16, 17, 18,   19, 20, 21, 24, 27, 29, 32, 35, 37, 40, 43, 45, 48, 56, 80];

		this.registerValueConverter(id, {
			convertValueOnGet : function( pixelValue ) {
				return myPoints[myPixel.indexOf(parseInt(pixelValue))];

			},
			convertValueOnSet : function( pointValue ) {
				return myPixel[myPoints.indexOf(parseFloat(pointValue))];
			}
		});

		this.addCompoBoxEntry(id,
			this.getValueLabelList(myPoints),
			tabId
		);
	};
};

TT_UI_Builder.prototype.customizing = {

    mappedCustomizings: {},

    initialize: function () {
    },

    getValue: function (id, key, defaultValue) {

        if( !id || !key ) {
            return defaultValue;
        }

		if ((typeof(this.mappedCustomizings[id]) === 'undefined') || (this.mappedCustomizings[id] === null)) {
			return defaultValue;
		}

		if ((typeof(this.mappedCustomizings[id][key]) === 'undefined') || (this.mappedCustomizings[id][key] === null)) {
			return defaultValue;
		}

		return this.mappedCustomizings[id][key];
    },

    addEntry: function (id, key, value) {

        if( !id || !key || !value ) {
            return this;
        }

        if ((typeof(this.mappedCustomizings[id]) === 'undefined') || (this.mappedCustomizings[id] === null)) {
            this.mappedCustomizings[id] = {};
        }

        this.mappedCustomizings[id][key] = value;

        return this;
    }
};

TT_UI_Builder.prototype.builder = {
    mappedTableBodies: {},
    tableBody: null,

    initialize: function (tableBodyId, guiTexts) {
        this.initializeTable(tableBodyId);
        this.textHandler.initialize(guiTexts);
    },

    textHandler: {

        currentLabels: null,
        defaultLanguage: "en-us",
        currentLanguage: "en-us",

        initialize: function (labels) {

            this.currentLanguage = getGuiLanguage();
            if (!this.currentLanguage) {
                this.currentLanguage = this.defaultLanguage;
            }

            this.currentLabels = labels;
        },

        getCaption: function (id) {

            var labelObject = this.getLabelObject(id);
            if (!labelObject) {
                return ( id );
            }

            return ( this.getLocalizedLabel(labelObject) );
        },

        getLabelObject: function (id) {

            if (!this.currentLabels) {
                return ( null);
            }

            var labels = this.currentLabels;
			if( id.indexOf( TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE + "." ) === 0 ) {
                labels = labels[TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE];
                id = id.substring(TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE.length + 1);
            }

            if (!labels) {
                return ( null);
            }

            if (typeof( labels[id] ) !== "undefined") {
                return ( labels[id] );
            }

            return ( null );
        },

        getLocalizedLabel: function (labelObject) {

            if (!labelObject) {
                return ( null);
            }

			if( typeof( labelObject[this.currentLanguage] ) !== "undefined" ) {
                return ( labelObject[this.currentLanguage] );
            }

			if( typeof( labelObject[this.defaultLanguage] ) !== "undefined" ) {
                return ( labelObject[this.defaultLanguage] );
            }

            return ( labelObject );
        }
    },

    initializeTable: function (id) {
        var map = id;
        if (typeof id === 'string') {
            map = {"default": id};
        }

        var builder = this;

        $.each(map, function (key, value) {

            builder.mappedTableBodies[key] = $("#" + value);
            if (key === 'default') {
                builder.tableBody = builder.mappedTableBodies[key];
            }

            if (!builder.mappedTableBodies[key]) {
                return ( false );
            }

            builder.mappedTableBodies[key].addClass("ParameterTable");

            var formatRow = builder.createRow();
            formatRow.css("display", "none");

            var formatCell = $(document.createElement("td"));
            formatCell.html("Name");
            formatCell.addClass("IdColCaption");
            formatRow.append(formatCell);

            formatCell = $(document.createElement("td"));
            formatCell.html("Value");
            formatCell.addClass("ValueColCaption");
            formatRow.append(formatCell);

            builder.mappedTableBodies[key].append(formatRow);
        });

        return ( true );
    },

    getTableBody: function (tabId) {

        if ((typeof(tabId) === 'undefined') || (tabId === null)) {
            return this.getTableBody("default");
        } else {
            if ((typeof(this.mappedTableBodies[tabId]) === 'undefined') || (this.mappedTableBodies[tabId] === null)) {
                return this.tableBody;
            } else {
                return this.mappedTableBodies[tabId];
            }
        }
    },

    createRow: function () {
        var row = $(document.createElement("tr"));
        return ( row );
    },

    createIdCell: function (id) {
        var cell = $(document.createElement("td"));
        cell.addClass("EntryId");
        cell.html(this.textHandler.getCaption(id));
        return ( cell );
    },

    createValueCell: function () {
        var cell = $(document.createElement("td"));
        cell.addClass("EntryValue");
        return cell;
    },

    createEditCell: function (id) {
        var cell = this.createValueCell(),
			cellDiv = $(document.createElement("div"));

        cellDiv.addClass("EntryValueLayout");
        cell.append(cellDiv);

        var editElement = $(document.createElement("input"));
        editElement.addClass("EntryValueEdit");
        editElement.attr("id", id);
        editElement.attr("type", "text");
        editElement.attr("value", ValueUtil.get(id));
        if(!this.isPermitted(id)){
            editElement.attr("disabled", "disabled");
        }
        editElement.bind("keyup", {"editId": id}, function (event) {
            var id = event.data.editId;
            var value = editElement.attr("value");
            ValueUtil.set(id, value);
        });

        cellDiv.append(editElement);

        return ( cell );
    },

    createMiniHeaderCell: function (values) {

        var cell = this.createValueCell();

        for (var i = 0; i < values.length; ++i) {
            var id = values[i].name;

            var cellDiv = $(document.createElement("div"));
            cellDiv.addClass("MiniEntryValueHeader");
            cellDiv.html(this.textHandler.getCaption(id));
            cell.append(cellDiv);
        }

        return ( cell );
    },

    createMiniEditCell: function (prefix, values) {

        var cell = this.createValueCell(),
            createMiniCell = function (id, callback, isPermitted) {
                var editElement = $(document.createElement("input"));
                editElement.addClass("MiniEntryValueEdit");
                editElement.attr("id", id);
                if(!isPermitted){
                    editElement.attr("disabled", "disabled");
                }
                editElement.attr("value", ValueUtil.get(id));
                editElement.bind("keyup", {"editId": id}, function (event) {
                    var id = event.data.editId;
                    var value = editElement.attr("value");
                    ValueUtil.set(id, value);
                    if( ! ( (typeof callback === null) || (callback === null) )){
                        callback(id, value);
                    }
                });
                return editElement;
            };

        for (var i = 0; i < values.length; ++i) {
            var id = prefix + values[i].name;

            var cellDiv = $(document.createElement("div"));
            cellDiv.addClass("EntryValueLayout");
            cell.append(cellDiv);
            cellDiv.append( createMiniCell( id, values[i].callback, this.isPermitted(id) ) );
        }

        return cell;
    },

    createTextAreaCell: function (id, rows, cols) {

        var cell = this.createValueCell();

        var cellDiv = $(document.createElement("div"));
        cellDiv.addClass("EntryValueLayout");
        cell.append(cellDiv);

        var editElement = $(document.createElement("textarea"));
        editElement.addClass("EntryValueTextArea");
        editElement.attr("id", id);
        editElement.attr("rows", rows);
        editElement.attr("cols", cols);
        if(!this.isPermitted(id)){
            editElement.attr("disabled", "disabled");
        }

        var value = ValueUtil.get(id);
        if (value) {
            value = value.replace(/\<br\/\>/g, "\n\r");
            editElement.attr("value", value);
        }

        editElement.bind("keyup", {"editId": id}, function (event) {
            var id = event.data.editId;
            var targetObject = $("#" + id);
            var value = targetObject.attr("value");
            if (value) {
                value = value.replace(/\n\r?/g, "<br/>");
                value = value.replace(/\r?/g, "");
            }
            ValueUtil.set(id, value);
        });

        cellDiv.append(editElement);

        return cell;
    },

    createComboBoxCell: function (id, comboList) {

        var cell = this.createValueCell();

        var cellDiv = $(document.createElement("div"));
        cellDiv.addClass("EntryValueLayout");
        cell.append(cellDiv);

        var valueSelectElement = $(document.createElement("select"));
        valueSelectElement.addClass("EntryValueComboBox");
        valueSelectElement.attr("id", id);
        if(!this.isPermitted(id)){
            valueSelectElement.attr("disabled", "disabled");
        }

        for (var i = 0; i < comboList.length; i++) {
            var value = comboList[i].value;
            var label = comboList[i].label;
            $("<option/>").val(value).text(label).appendTo(valueSelectElement);
        }

        valueSelectElement.attr("value", ValueUtil.get(id));
        valueSelectElement.bind("change", {"editId": id}, function (event) {
            var id = event.data.editId;
            var targetObject = $("#" + id);
            var value = targetObject.val();
            ValueUtil.set(id, value);
        });

        cellDiv.append(valueSelectElement);

        return cell;
    },

    createLocalizedComboBoxCell: function (id, comboList) {

        var cell = this.createValueCell();

        var cellDiv = $(document.createElement("div"));
        cellDiv.addClass("EntryValueLayout");
        cell.append(cellDiv);

        var valueSelectElement = $(document.createElement("select"));
        valueSelectElement.addClass("EntryValueComboBox");
        valueSelectElement.attr("id", id);

        for (var i = 0; i < comboList.length; i++) {
            var value = comboList[i].value;
            var label = this.textHandler.getLocalizedLabel( this.textHandler.getLabelObject(comboList[i].key) );
            $("<option/>").val(value).text(label).appendTo(valueSelectElement);
        }

        valueSelectElement.attr("value", ValueUtil.get(id));
        valueSelectElement.bind("change", {"editId": id}, function (event) {
            var id = event.data.editId;
            var targetObject = $("#" + id);
            var value = targetObject.val();
            ValueUtil.set(id, value);
        });

        cellDiv.append(valueSelectElement);

        return cell;
    },

    createCheckBoxCell: function (id) {

        var cell = this.createValueCell();

        var cellDiv = $(document.createElement("div"));
        cellDiv.addClass("EntryValueLayout");
        cell.append(cellDiv);

        var checkBox = $(document.createElement("input"));
        checkBox.addClass("EntryValueCheckBox");
        checkBox.attr("id", id);
        checkBox.attr("type", "checkbox");
        if(!this.isPermitted(id)){
            checkBox.attr("disabled", "disabled");
        }

        var value = ValueUtil.get(id);
        if (value) {
            checkBox.attr("checked", (value.toLowerCase() === "true"));
        }

        checkBox.bind("mouseup", {"editId": id}, function (event) {
            var id = event.data.editId;
            var targetObject = $("#" + id);
            var value = targetObject.attr("checked");
            ValueUtil.set(id, value ? "false" : "true");
        });

        cellDiv.append(checkBox);

        return cell;
    },

    createFileSelectionCell: function (id, hideInfoButton) {

        var editElement = $(document.createElement("input")),
            cell = this.createValueCell(),
            cellDiv = $(document.createElement("div"));

        cellDiv.addClass("EntryValueLayout");
        cell.append(cellDiv);

        editElement.addClass("EntryValueFileSelection");
        editElement.attr("id", id);
        editElement.attr("value", ValueUtil.get(id));
		editElement.prop("disabled", "disabled");
        editElement.bind("keyup", {"editId": id}, function (event) {

            var id = event.data.editId,
                editElement = $("#" + id),
                value;

            if (editElement) {
                value = editElement.attr("value");
                if (!value) {
                    value = "";
                }
                ValueUtil.set(id, value);
            }
        });

        cellDiv.append(editElement);

        var isActive = this.isPermitted(id),
            importButton = this.createFileSelectionButton(id, isActive),
            trashButton = this.createTrashButton(id, isActive);

        cellDiv.append(importButton);

        if(!isActive){
            importButton.button( "option", "disabled", true );
            trashButton.button( "option", "disabled", true );
        }
        cellDiv.append(trashButton);

		if( !hideInfoButton ){
			var infoButton = this.createInfoButton(id, isActive);
			cellDiv.append(infoButton);
		}

        return cell;
    },

    createFileSelectionButton: function (id, isActive) {

        var importFileButton = $(document.createElement("span"));
        importFileButton.attr("id", "importFile_" + id);
        importFileButton.addClass("ui-state-default");
        importFileButton.addClass("ui-corner-all");
        importFileButton.addClass("EntryControlButton");

        var importFileIcon = $(document.createElement("span"));
        importFileIcon.addClass("ui-icon");
        importFileIcon.addClass("ui-icon-folder-open");
        importFileButton.append(importFileIcon);

        if(isActive){
            importFileButton.bind("mouseup", {"editId": id}, function (event) {

                var id = event.data.editId,
                    filePath = importFile(id),
                    editElement = $("#" + id);

                if( filePath === "" ){
                    // cancel button pressed
                    return;
                }

                $(window.document).trigger( "file:selected", {
                    "id" : id,
                    "filePath" : filePath
                });

                if( editElement ) {
                    editElement.attr("value", filePath);
                }

                ValueUtil.set(id, filePath);
            });
        }

        importFileButton.button( );

        return ( importFileButton );
    },

    createTrashButton: function (id, isActive) {

        var trashButton = $(document.createElement("span"));
        trashButton.attr("id", "trash_" + id);
        trashButton.addClass("ui-state-default");
        trashButton.addClass("ui-corner-all");
        trashButton.addClass("EntryControlButton");

        var trashIcon = $(document.createElement("span"));
        trashIcon.addClass("ui-icon");
        trashIcon.addClass("ui-icon-trash");
        trashButton.append(trashIcon);

        if(isActive) {
            trashButton.bind(
                "mouseup",
                {"editId": id},
                function (event) {

                    var id = event.data.editId,
                        editElement = $("#" + id);

                    $(window.document).trigger( "file:selected", {
                        "id" : id,
                        "filePath" : ""
                    });

                    if (editElement) {
                        editElement.attr("value", "");
                    }

                    ValueUtil.set(id, "");
                }
            );
        }

        trashButton.button( );

        return ( trashButton );
    },

    createInfoButton: function (id, isActive) {

        var url = 'help/' + id + '.' + getGuiLanguage() + '.htm',
		    urlEN = 'help/' + id + '.en-us.htm',
		    infoButton = $(document.createElement("span"));

        infoButton.attr("id", "info_" + id);
        infoButton.addClass("ui-state-default");
        infoButton.addClass("ui-corner-all");
        infoButton.addClass("EntryControlButton");

        var infoIcon = $(document.createElement("span"));
        infoIcon.addClass("ui-icon");
        infoIcon.addClass("ui-icon-info");
        infoButton.append(infoIcon);

		// check if help file exists
		$.ajaxSetup({
			xhr: function()  {
				if ('ActiveXObject' in window) {
					return new ActiveXObject("Microsoft.XMLHTTP");
				} else {
					return new XMLHttpRequest();
				}
			}
		});

		$.get(url).success(function () {
			if(isActive){
				infoButton.bind("mouseup", {"url": url}, function() {
					window.open(url);
				});
			}
        }).error(function() {
			if(isActive){
				infoButton.bind("mouseup", {"url": urlEN}, function() {
					window.open(urlEN);
				});
			}
        });

		infoButton.button();

		return infoButton;
    },

    createCustomCell: function (id, element) {
        if (!element) {
            return;
        }

        var cell = this.createValueCell(),
            cellDiv = $(document.createElement("div"));

        cellDiv.addClass("EntryValueLayout");
        cell.append(cellDiv);

        cellDiv.append(element);

        return cell;
    },

    createSlider: function (id) {
		var cell = this.createValueCell();

		var cellDiv = $(document.createElement("div"));
		cellDiv.addClass("sliderSingle").attr({
			'id': id,
			'permitted': this.isPermitted(id.replace("_slider", ""))
		});
		cell.append(cellDiv);

        return cell;
    },

    createRangeSlider: function (id) {
        var cell = this.createValueCell();

        var cellDiv = $(document.createElement("div"));
        cellDiv.addClass("sliderRanged").attr({
            'id': id
        });
        cell.append(cellDiv);

        return cell;
    },

    createEntry: function (tabId, id, idCell, valueCell, descriptionCell) {
        var tableBody = this.getTableBody(tabId);

        if (!tableBody) {
            return;
        }

        var entryRow = this.createRow();
        if (!idCell || !valueCell) {
            return;
        }

        entryRow.append(idCell);
        entryRow.append(valueCell);

        tableBody.append(entryRow);

        if (descriptionCell) {
            var infoRow = this.createRow();
            tableBody.append(infoRow);
        }
    },

    createEntryDescription: function (tabId, id) {
        var tableBody = this.getTableBody(tabId);

        if (!tableBody) {
            return;
        }

        var description = this.textHandler.getCaption(id);
		if( !description || description === "" ) {
            return;
        }

        var row = this.createRow();

        var descriptionIdCellElement = $(document.createElement("td"));
        descriptionIdCellElement.addClass("EntryDescriptionId");
        row.append(descriptionIdCellElement);

        var descriptionCellElement = $(document.createElement("td"));
        descriptionCellElement.addClass("EntryDescriptionValue");
        descriptionCellElement.html(description);

        row.append(descriptionCellElement);

        tableBody.append(row);
    },

	// ein neue Funktion zum hinzufügen einer Beschreibung,
	// die sich über die komplette Parametertabelle erstreckt.
    createDescription: function (tabId, id) {
        var tableBody = this.getTableBody(tabId);

        if (!tableBody) {
            return;
        }

        var description = this.textHandler.getCaption(id);
		if( !description || description === "" ) {
            return;
        }

        var row = this.createRow();

        var descriptionCellElement = $(document.createElement("td"));
        descriptionCellElement.addClass("tabDescription");
        descriptionCellElement.attr({
            "id" : id,
            "colspan" : "2"
        });
        descriptionCellElement.html(description);

        row.append(descriptionCellElement);

        tableBody.append(row);
    },

    isPermitted : function(id) {

        var allPermitted = [];
        try {
			allPermitted = $.parseJSON(ValueUtil.getList());
		}
		catch(e) {
            console.error( "Error while parsing value list: " + ValueUtil.getList() );
			return false;
		}


        for (var i=0; i<allPermitted.length; i++) {
            if (allPermitted[i] === id) {
                return true;
            }
        }
        return false;
    },

    createSeperator: function (tabId) {
        var tableBody = this.getTableBody(tabId);

        if (!tableBody) {
            return;
        }

        var row = this.createRow();
        var seperatorCellElement = $(document.createElement("td"));
        seperatorCellElement.attr("colSpan", 2);
        seperatorCellElement.addClass("Seperator");

        row.append(seperatorCellElement);

        tableBody.append(row);
    }
};

TT_UI_Builder.AUTHOR_PARAMETER_NAMESPACE = "com.tts";
